import { rootNode } from "@ark/schema";
import { scope } from "../scope.js";
import { creditCardMatcher, isLuhnValid } from "./utils/creditCard.js";
import { ip } from "./utils/ip.js";
import { regexStringNode } from "./utils/regex.js";
// Non-trivial expressions should have an explanation or attribution
const url = rootNode({
    domain: "string",
    predicate: {
        meta: "a valid URL",
        predicate: (s) => {
            try {
                new URL(s);
            }
            catch {
                return false;
            }
            return true;
        }
    }
});
// https://www.regular-expressions.info/email.html
const emailMatcher = /^[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Za-z]{2,}$/;
const email = regexStringNode(emailMatcher, "a valid email");
const uuidMatcher = /^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{12}$/;
// https://github.com/validatorjs/validator.js/blob/master/src/lib/isUUID.js
const uuid = regexStringNode(uuidMatcher, "a valid UUID");
const semverMatcher = /^(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$/;
// https://semver.org/
const semver = regexStringNode(semverMatcher, "a valid semantic version (see https://semver.org/)");
const creditCard = rootNode({
    domain: "string",
    pattern: {
        meta: "a valid credit card number",
        rule: creditCardMatcher.source
    },
    predicate: {
        meta: "a valid credit card number",
        predicate: isLuhnValid
    }
});
/**
 * As per the ECMA-262 specification:
 * A time value supports a slightly smaller range of -8,640,000,000,000,000 to 8,640,000,000,000,000 milliseconds.
 *
 * @see https://262.ecma-international.org/15.0/index.html#sec-time-values-and-time-range
 */
const unixTimestamp = rootNode({
    domain: {
        domain: "number",
        meta: "a number representing a Unix timestamp"
    },
    divisor: {
        rule: 1,
        meta: `an integer representing a Unix timestamp`
    },
    min: {
        rule: -8640000000000000,
        meta: `a Unix timestamp after -8640000000000000`
    },
    max: {
        rule: 8640000000000000,
        meta: "a Unix timestamp before 8640000000000000"
    },
    meta: "an integer representing a safe Unix timestamp"
});
export const validation = scope({
    alpha: regexStringNode(/^[A-Za-z]*$/, "only letters"),
    alphanumeric: regexStringNode(/^[A-Za-z\d]*$/, "only letters and digits 0-9"),
    digits: regexStringNode(/^\d*$/, "only digits 0-9"),
    lowercase: regexStringNode(/^[a-z]*$/, "only lowercase letters"),
    uppercase: regexStringNode(/^[A-Z]*$/, "only uppercase letters"),
    creditCard,
    email,
    uuid,
    url,
    semver,
    ip,
    integer: rootNode({
        domain: "number",
        divisor: 1
    }),
    unixTimestamp
}, { prereducedAliases: true }).export();
